package com.zhongshu.reward.server.core.service;

import com.github.microservice.core.util.result.content.ResultContent;
import com.github.microservice.core.util.result.content.ResultState;
import com.zhongshu.reward.client.model.param.WalletReceiptsParam;
import com.zhongshu.reward.client.model.wallet.WalletReceiptsModel;
import com.zhongshu.reward.client.type.ReceiptsStatus;
import com.zhongshu.reward.server.core.dao.WalletDao;
import com.zhongshu.reward.server.core.dao.WalletReceiptsDao;
import com.zhongshu.reward.server.core.domain.Wallet;
import com.zhongshu.reward.server.core.domain.WalletReceipts;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author wjf
 * @date 2024/8/6
 */
@Service
@Slf4j
public class WalletReceiptsService {

    @Autowired
    WalletDao walletDao;

    @Autowired
    WalletService walletService;

    @Autowired
    WalletReceiptsDao walletReceiptsDao;

    /**
     * 邀请返利入账
     * @param param
     * @return
     */
    @Transactional
    public Object receipts(WalletReceiptsParam param){

        Wallet wallet = walletService.getWalletByUserId(param.getUserId());

        WalletReceipts walletReceipts = new WalletReceipts();
        BeanUtils.copyProperties(param, walletReceipts);
        walletReceipts.setWallet(wallet);
        if (walletReceipts.getReceiptsType()==null){
            walletReceipts.setStatus(ReceiptsStatus.WAIT);
        }
        wallet.setWaitAmount(wallet.getWaitAmount().add(param.getTotal()));
        walletDao.save(wallet);
        return ResultContent.buildContent(toModel(walletReceiptsDao.save(walletReceipts)));
    }

    /**
     * 取消返利入账
     * @param
     * @return
     */
    @Transactional
    public Object cancelReceipts(String outTradeNo){

        WalletReceipts walletReceipts = walletReceiptsDao.findTopByOutTradeNo(outTradeNo);
        if (ObjectUtils.isEmpty(walletReceipts)){
            return ResultContent.build(ResultState.Fail,"返利订单不存在或未入账");
        }
        if (!walletReceipts.getStatus().equals(ReceiptsStatus.WAIT)){
            return ResultContent.build(ResultState.Fail,"订单不处于未结算状态");
        }

        walletReceipts.setStatus(ReceiptsStatus.CANCEL);
        walletReceiptsDao.save(walletReceipts);
        Wallet wallet = walletDao.findTop1ById(walletReceipts.getWallet().getId());
        wallet.setWaitAmount(wallet.getWaitAmount().subtract(walletReceipts.getTotal()));
        walletDao.save(wallet);
        return ResultContent.build(ResultState.Success);
    }

    public Object listByWallet(String walletId){
        List<WalletReceipts> list = walletReceiptsDao.findByWallet_IdOrderByCreateTimeDesc(walletId);
        return ResultContent.buildContent(list.stream().map(this::toModel).collect(Collectors.toList()));
    }

    WalletReceiptsModel toModel(WalletReceipts walletReceipts){
        WalletReceiptsModel model = new WalletReceiptsModel();
        if (null != walletReceipts){
            BeanUtils.copyProperties(walletReceipts, model, "wallet");
            model.setWalletId(walletReceipts.getWallet().getId());
        }
        return model;
    }
}
